import 'lebensbereiche.dart';
import 'dart:io';
import 'package:ansicolor/ansicolor.dart';

int menue_index, runde;
var gameOver = false;
double highscore;

List<String> menue_entries = [
  'Sanierung',
  'Produktion',
  'Aufklärung',
  'Lebensqualität',
  'Vermehrungsrate',
  'Umweltbelastung',
  'Bevölkerung',
  'Politik',
  'Versorgungsfaktor',
  'Wachstumsfaktor',
  'Aktionspunkte',
];

void clearScreen() => print('\x1B[2J\x1B[0;0H');
void resetCursor() => print('\x1B[0;0H');
void tab() => stdout.write('\t');
void arrowDown() {
  if (menue_index <= 7) {
    menue_index += 4;
  }
}

void arrowUp() {
  if (menue_index >= 4) {
    menue_index -= 4;
  }
}

void arrowLeft() {
  if (menue_index > 0) {
    menue_index -= 1;
  }
}

void arrowRight() {
  if (menue_index < 11) {
    menue_index += 1;
  }
}

void rundeBeenden() {
  lebensbereiche.calculateChanges();
  if (lebensbereiche.checkForStoppedParams()) {
    final bilanzZahl = getBilanzzahl(runde);
    if (highscore == null || highscore < bilanzZahl) {
      highscore = bilanzZahl;
    }
    gameOver = true;
  } else {
    runde += 1;
  }
}

// 0 = normal, -1 = quit
int handleInput(key) {
  if (key == 'q') return -1;
  if (key == 'ControlCharacter.arrowDown' || key == 's' || key == '2') {
    arrowDown();
  } else if (key == 'ControlCharacter.arrowUp' || key == 'w' || key == '8') {
    arrowUp();
  } else if (key == 'ControlCharacter.arrowLeft' || key == 'a' || key == '4') {
    arrowLeft();
  } else if (key == 'ControlCharacter.arrowRight' || key == 'd' || key == '6') {
    arrowRight();
  } else if (key == 'ControlCharacter.enter') {
    if (menue_index >= 0 && menue_index <= 4) {
      lebensbereiche.changeValue('Aktionspunkte', -1);
      lebensbereiche.changeValue(menue_entries[menue_index], 1);
      if (lebensbereiche.getValue('Aktionspunkte') == 0) {
        rundeBeenden();
      }
    }
  } else if (key == 'ControlCharacter.tab') {
    lebensbereiche.erhoehenBeiHoherAufklaerung =
        !lebensbereiche.erhoehenBeiHoherAufklaerung;
  }
  return 0;
}

String ueberschrift(String str) {
  final pen = AnsiPen()..rgb(r: 1.0, g: 0.8, b: 0.2);
  return pen(str);
}

String markiert(String str) {
  final pen = AnsiPen()..white(bg: true);
  return pen(str);
}

String blau(String str) {
  final pen = AnsiPen()..blue();
  return pen(str);
}

String magenta(String str) {
  final pen = AnsiPen()..magenta();
  return pen(str);
}

String gruen(String str) {
  final pen = AnsiPen()..green();
  return pen(str);
}

String rot(String str) {
  final pen = AnsiPen()..red();
  return pen(str);
}

void printParam(String str, {Function farbe}) {
  final valStr = lebensbereiche.getValue(str).toString();
  var outputText = farbe(str + ': ' + valStr);
  final toBeMarked = menue_index == menue_entries.indexOf(str);
  stdout.write(toBeMarked ? markiert(outputText) : outputText);
  stdout.write('\t\t');
}

void erklaerungsText() {
  if (!gameOver) {
    print(rot('Enter:') +
        ' 1 Aktionspunkt auf ' +
        gruen('grünem') +
        ' Feld verteilen.');
    print(rot('Tab:') +
        ' Bei hoher Aufklärung Vermehrungsrate erhöhen/verringern.');
  } else {
    print(rot('R:') + ' Nochmals spielen');
    print(rot('Q:') + ' Spiel verlassen');
  }
  String highscoreStr;
  if (highscore == null) {
    highscoreStr = 'unbekannt';
  } else {
    highscoreStr = highscore.toString();
  }
  print(blau('Highscore:') + ' ' + highscoreStr);
}

void printGameboard() {
  clearScreen();
  final bilanzZahlStr = getBilanzzahlStr(runde);

  if (!gameOver) {
    stdout.write(ueberschrift('\t\t\t\t\t\tKybernetisches Planspiel'));
    stdout.write(ueberschrift(' (Runde ' + runde.toString() + ')\t'));
    print(
        ueberschrift((lebensbereiche.erhoehenBeiHoherAufklaerung ? '+' : '-')) +
            '\n');
  } else {
    print(ueberschrift('\t\t\t\t\t\tExtremzustand erreicht! Schlußbilanz: ' +
            bilanzZahlStr) +
        '\n');
  }
  printParam(menue_entries[0], farbe: gruen);
  tab();
  tab();
  printParam(menue_entries[1], farbe: gruen);
  tab();
  printParam(menue_entries[2], farbe: gruen);
  tab();
  printParam(menue_entries[3], farbe: gruen);
  print('\n');
  printParam(menue_entries[4], farbe: gruen);
  tab();
  printParam(menue_entries[5], farbe: magenta);
  printParam(menue_entries[6], farbe: magenta);
  tab();
  printParam(menue_entries[7], farbe: magenta);
  print('\n');
  printParam(menue_entries[8], farbe: magenta);
  tab();
  printParam(menue_entries[9], farbe: magenta);
  printParam(menue_entries[10], farbe: blau);
  final bilanzText = blau('Bilanzzahl: ' + bilanzZahlStr);
  print(menue_index == 11 ? markiert(bilanzText) : bilanzText);
  print('\n');
  erklaerungsText();
  resetCursor();
}

void initializeGame() {
  clearScreen();
  stdin.echoMode = false;
  stdin.lineMode = false;
  lebensbereiche.resetValues();
  gameOver = false;
  menue_index = 0;
  runde = 1;
}
