import 'params/param.dart';
import 'params/sanierung.dart';
import 'params/produktion.dart';
import 'params/umweltbelastung.dart';
import 'params/aufklaerung.dart';
import 'params/lebensqualitaet.dart';
import 'params/vermehrungsrate.dart';
import 'params/bevoelkerung.dart';
import 'params/politik.dart';

class Lebensbereiche {
  Map<String, Param> _bereiche;
  bool erhoehenBeiHoherAufklaerung = true;

  Lebensbereiche() {
    _bereiche = {
      'Sanierung': sanierung,
      'Produktion': produktion,
      'Umweltbelastung': umweltbelastung,
      'Aufklärung': aufklaerung,
      'Lebensqualität': lebensqualitaet,
      'Vermehrungsrate': vermehrungsrate,
      'Bevölkerung': bevoelkerung,
      'Politik': politik,
      'Aktionspunkte': Param(8, 36, [], minimalWert: 0),
      'Versorgungsfaktor': Param(1, 1, [], minimalWert: -5),
      'Wachstumsfaktor': Param(2, 3, [], minimalWert: 0),
      'A': Param(2, 48, [], minimalWert: 0),
      'B': Param(0, 3, [], minimalWert: -5),
      'C': Param(4, 11, [], minimalWert: -4),
      'D': Param(2, 5, [], minimalWert: -6)
    };
  }

  int getValue(String str) => _bereiche[str].aktuellerWert;

  void changeValue(String str, int difference) {
    _bereiche[str].addPoints(difference);
  }

  void calculateChanges() {
    [
      'Sanierung',
      'Produktion',
      'Aufklärung',
      'Lebensqualität',
      'Vermehrungsrate'
    ].forEach((String str) {
      final changes = _bereiche[str].changes();
      for (final entry in changes.entries) {
        final name = entry.key; // Der Name des zu verändernden Parameters
        var wert = entry.value;
        if (name == 'Vermehrungsrate' &&
            _bereiche['Aufklärung'].aktuellerWert >= 21 &&
            !erhoehenBeiHoherAufklaerung) {
          wert = -entry.value;
        }

        if (name == 'Bevölkerung') {
          wert = _bereiche['Wachstumsfaktor'].aktuellerWert * entry.value;
        }

        if (name.length == 1 || name.contains('faktor')) {
          // Rote Fenster und Wachstumsfaktor (sozusagen die "indirekten Fenster")
          _bereiche[entry.key].setPoints(wert);
        } else {
          _bereiche[entry.key].addPoints(wert);
        }
      }
    });
    _bereiche['Aktionspunkte'].addPoints(
        _bereiche['Versorgungsfaktor'].aktuellerWert *
                _bereiche['A'].aktuellerWert +
            _bereiche['B'].aktuellerWert +
            _bereiche['C'].aktuellerWert +
            _bereiche['D'].aktuellerWert);
  }

  bool checkForStoppedParams() {
    for (final key in _bereiche.keys) {
      if (_bereiche[key].isStopped()) {
        print(key);
        return true;
      }
    }
    return false;
  }

  void resetValues() {
    for (final param in _bereiche.values) {
      param.reset();
    }
  }
}

final lebensbereiche = Lebensbereiche();

double getBilanzzahl(int spielRunden) {
  final ergebnis1 = 10 *
      (lebensbereiche.getValue('Politik') + 3 * lebensbereiche.getValue('D'));
  final ergebnis2 = spielRunden + 3;
  return double.parse((ergebnis1 / ergebnis2).toStringAsFixed(2));
}

String getBilanzzahlStr(int spielRunden) {
  var bilanzZahlStr = getBilanzzahl(spielRunden).toString();
  if (bilanzZahlStr[bilanzZahlStr.length - 1] == '0') {
    bilanzZahlStr = bilanzZahlStr.substring(0, bilanzZahlStr.length - 2);
  }
  return bilanzZahlStr;
}
